#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Union, Callable, Optional, Tuple, Dict, List, Generator, Any

from oser.core import Array as _Array
from oser.types import OserElementType, OserStructType


class Array(_Array):
    """
    ``Array`` builds an array serializer with a fixed or variable length.
    """

    def __init__(
        self,
        length: Union[Callable, int],
        prototype: OserElementType,
        args: Optional[Tuple] = None,
        kwargs: Optional[Dict] = None,
        values: Optional[List[OserElementType]] = None,
    ) -> None:
        """
        Args:
            length: states the array length. Can be a callable (e.g. lambda) or a scalar.
            prototype: a class that is the prototype for `values`
            args=None: a tuple of positional arguments that are passed to `prototype` when it is instantiated
                when array size is extended.
                Note: don't forget a leading comma when passing one value ``(1,)`` (``(1)`` is an integer!).
            kwargs=None: a dictionary of named arguments that are passed to `prototype` when it is instantiated
                when array size is extended.
            values=None: the initial array values. Can be left empty.
                If empty the array is automatically resized with instances of ``prototype(*args, **kwargs)``.
        """
        super(Array, self).__init__(length=length, prototype=prototype, args=args, kwargs=kwargs, values=values)

    def get_byte_size(self) -> int:  # type: ignore[override]
        """
        Return the current size in bytes.
        """
        return super(Array, self).get_byte_size()

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(Array, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(Array, self).encode(full_data=full_data, context_data=context_data)

    def from_dict(self, data: Dict) -> None:
        """
        Fill self with data.

        Args:
            data: data to be used to fill the calling instance.
        """
        return super(Array, self).from_dict(data=data)

    def fuzzing_iterator(self, copy: bool = False) -> Generator[Any, None, None]:
        """
        The fuzzing iterator iterates over all combinations
        of set fuzzing values (:func:`oser.ByteType.set_fuzzing_values`).
        If no fuzzing values are set the current struct is yielded.

        Args:
            copy=False: if set to ``True`` the generated
                fuzzing values are deep copies of the original.
                Creating these deep copies is slow. If set to ``False`` the
                original struct is retruned and the generated value must be
                used immediately since the next generated overwrites the values
                on the same value.

        Yields:
            the fuzzing combinations.
        """
        return super(Array, self).fuzzing_iterator(copy=copy)

    def get(self) -> "Array":
        """
        Return the value.

        Returns:
            the value
        """
        return super(Array, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(Array, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(Array, self).root()

    def get_size(self) -> int:
        """
        Return the current size in bytes or bits according to the context.
        """
        return super(Array, self).get_size()

    def to_dict(self) -> List[Dict[Any, Any]]:  # type: ignore
        """
        Return self as a dict()
        """
        return super(Array, self).to_dict()

    def up(self) -> OserStructType:
        """
        return parent element
        """
        return super(Array, self).up()
