#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional

from oser.core import CRCB16 as _CRCB16
from oser.core import CRCB32 as _CRCB32
from oser.core import CRCB64 as _CRCB64
from oser.core import CRCB8 as _CRCB8
from oser.core import CRCL16 as _CRCL16
from oser.core import CRCL32 as _CRCL32
from oser.core import CRCL64 as _CRCL64
from oser.core import CRCL8 as _CRCL8
from oser.core import ChecksumMixin as _ChecksumMixin
from oser.types import OserElementType, OserStructType, FuzzingValues


class CRCB16(_CRCB16):
    """
    16-bit crc big endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 41003,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0xa02b: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x0000: start with `initial_value`
            xor_output: int = 0x0000: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCB16, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(CRCB16, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(CRCB16, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCB16, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCB16, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCB16, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCB16, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCB16, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCB16, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCB16, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCB16, self).up()


class CRCB32(_CRCB32):
    """
    32-bit crc big endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 517762881,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0x1edc6f41: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x00000000: start with `initial_value`
            xor_output: int = 0x00000000: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCB32, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(CRCB32, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(CRCB32, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCB32, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCB32, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCB32, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCB32, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCB32, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCB32, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCB32, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCB32, self).up()


class CRCB64(_CRCB64):
    """
    64-bit crc big endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 4823603603198064275,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0x42f0e1eba9ea3693: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x0000000000000000: start with `initial_value`
            xor_output: int = 0x0000000000000000: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCB64, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(CRCB64, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(CRCB64, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCB64, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCB64, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCB64, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCB64, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCB64, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCB64, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCB64, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCB64, self).up()


class CRCB8(_CRCB8):
    """
    8-bit crc big endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 213,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0xd5: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x00: start with `initial_value`
            xor_output: int = 0x00: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCB8, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.
        """
        return super(CRCB8, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(CRCB8, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCB8, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCB8, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCB8, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCB8, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCB8, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCB8, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCB8, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCB8, self).up()


class CRCL16(_CRCL16):
    """
    16-bit crc little endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 41003,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0xa02b: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x0000: start with `initial_value`
            xor_output: int = 0x0000: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCL16, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(CRCL16, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(CRCL16, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCL16, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCL16, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCL16, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCL16, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCL16, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCL16, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCL16, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCL16, self).up()


class CRCL32(_CRCL32):
    """
    32-bit crc little endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 517762881,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0x1edc6f41: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x00000000: start with `initial_value`
            xor_output: int = 0x00000000: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCL32, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(CRCL32, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(CRCL32, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCL32, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCL32, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCL32, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCL32, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCL32, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCL32, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCL32, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCL32, self).up()


class CRCL64(_CRCL64):
    """
    64-bit crc little endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 4823603603198064275,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0x42f0e1eba9ea3693: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x0000000000000000: start with `initial_value`
            xor_output: int = 0x0000000000000000: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCL64, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(CRCL64, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(CRCL64, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCL64, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCL64, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCL64, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCL64, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCL64, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCL64, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCL64, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCL64, self).up()


class CRCL8(_CRCL8):
    """
    8-bit crc little endian serializer.
    """

    def __init__(
        self,
        strict: bool = False,
        polynomial: int = 213,
        reflect_input: bool = False,
        reflect_output: bool = False,
        initial_value: int = 0,
        xor_output: int = 0,
        automatic_calculation: bool = True,
        format: str = "hex",
    ) -> None:
        """
        Args:
            strict: bool = False: If set to ``True`` crc is checked when decoding data.
                An exception is raised if crcs do not match.
            polynomial: int = 0xd5: the crc polynomial
            reflect_input: bool = False: reflect input data
            reflect_output: bool = False: reflect output data
            initial_value: int = 0x00: start with `initial_value`
            xor_output: int = 0x00: xor output data with `xor_output`
            automatic_calculation: bool = True: if set to ``True`` crc is automatically calculated when encoding data
            format: str = "hex" (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary. Use ``None`` for decimal format.
        """
        super(CRCL8, self).__init__(
            strict=strict,
            polynomial=polynomial,
            reflect_input=reflect_input,
            reflect_output=reflect_output,
            initial_value=initial_value,
            xor_output=xor_output,
            automatic_calculation=automatic_calculation,
            format=format,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(CRCL8, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded byte type as a binary string.

        Args:
            full_data (bytes): the binary data string until the part to be encoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.
        """
        return super(CRCL8, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(CRCL8, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(CRCL8, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(CRCL8, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(CRCL8, self).set(value=value)  # type: ignore

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(CRCL8, self).set_automatic_calculation(enabled=enabled)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(CRCL8, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(CRCL8, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(CRCL8, self).up()


class ChecksumMixin(_ChecksumMixin):  # pragma: no cover
    """
    Use the static methods encode and decode in your implementation
    and implement _calculate_checksum(data) that returns a calculated checksum.
    """

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string into a byte type and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.
        """
        return super(ChecksumMixin, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded byte type as a binary string.

        Args:
            full_data (bytes): the binary data string until the part to be encoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.
        """
        return super(ChecksumMixin, self).encode(full_data=full_data, context_data=context_data)

    def set_automatic_calculation(self, enabled: bool = True) -> None:
        """
        Enable or disable automatic calculation when ``encode`` is called.

        Args:
            enabled=True (bool): the new enabled state.
        """
        return super(ChecksumMixin, self).set_automatic_calculation(enabled=enabled)
