#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional, Union, Callable

from oser.core import Data as _Data
from oser.types import FuzzingValues
from oser.types import OserElementType, OserStructType


class Data(_Data):
    """
    ``Data`` builds a data serializer with fixed or variable length.
    Extends :class:`oser.String` but length must not be ``None``.
    """

    def __init__(
        self, length: Union[Callable, int], value: Union[bytes, str] = b"", padding: Union[bytes, str] = b"\x00"
    ) -> None:
        """
        Args:
            length: states the data length. Must be a callable (e.g. lambda) or an integer.
            value="": the initial data value.
            padding=b"\x00": that padding pattern used for filling the encoded data if `value` is shorter than `length`.
        """
        super(Data, self).__init__(length=length, value=value, padding=padding)

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded. The user normally does not need to
                supply this.
            context_data (bytes): the binary data of the current context. The user normally does not need to supply
                this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(Data, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(Data, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> bytes:
        """
        Return the value.
        """
        return super(Data, self).get()

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(Data, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(Data, self).root()

    def set(self, value: Union[bytes, str]) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(Data, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(Data, self).set_fuzzing_values(values=values)

    def set_length(self, length: Optional[Union[Callable, int]]) -> None:
        """
        Set the length.

        Args:
            length=None: states the string length. Can be a callable (e.g. lambda) or a scalar.
                If set to ``None`` the result is a null-terminated string.
                If set to an Integer the result is a fixed length string padded with ``padding``.
                If set to a ``callable`` (lambda or function) the result is a fixed or variable
                length string padded with ``padding``.
        """
        return super(Data, self).set_length(length=length)

    def get_size(self) -> int:
        """
        Return the size in bytes.
        """
        return super(Data, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(Data, self).up()
